@@ -1,13 +1,16 @@
 import copy
 import json
+import logging
 from typing import Union, List, Tuple, Any, Optional, Dict, Set, Type, TypeVar
 from collections.abc import MutableMapping
 import dotdict_parser
 
 T = TypeVar("T", bound='DotDict')
+logger = logging.getLogger(__name__)
 
 class DotDict(MutableMapping):
-    def __init__(self, dictionary: Optional[Union[dict | list]] = None):
+    def __init__(self, dictionary: Optional[Union[dict | list]] = None, override_data=False):
+        self._override_data = override_data
         if dictionary is None:
             dictionary = {}
         if not isinstance(dictionary, (dict, list)):
@@ -25,14 +28,22 @@ class DotDict(MutableMapping):
         for i in range(len(path) - 1):
             key = path[i]
             next_key = path[i + 1]
+            prev_key = path[i - 1] if i > 0 else "root"
 
             if isinstance(key, str):
                 # Ensure `node` is a dict if we are using a string key
                 if not isinstance(node, dict):
-                    raise TypeError(f"Cannot use string key on non-dict: {node}")
+                    raise TypeError(f"Cannot embed key '{key}' in '{prev_key}' (path: {prev_key}.{key}) because it is not a dict but a string: {{{prev_key}:{node}}}")
+
                 # If key doesn't exist, create either a dict or list based on the next key
                 if key not in node:
                     node[key] = [] if isinstance(next_key, int) else {}
+                elif self._override_data:
+                    # key in node but not correct type
+                    if not isinstance(node[key], dict):
+                        logger.info(f"Path: {path} overrides old data, because '{key}' it is not a dict but a {type(node[key])} = {node[key]}")
+                        node[key] = [] if isinstance(next_key, int) else {}
+
                 node = node[key]
 
             elif isinstance(key, int):
@@ -58,7 +69,7 @@ class DotDict(MutableMapping):
         last_key = path[-1]
         if isinstance(last_key, str):
             if not isinstance(node, dict):
-                raise TypeError(f"Cannot assign string-key '{last_key}' to non-dict: {node}")
+                raise TypeError(f"Cannot change '{'.'.join(path)}' because some of nodes {path} are no-dict: {last_key}. Set override_data=True (current:{self._override_data}) to allow this.")
             node[last_key] = value
         elif isinstance(last_key, int):
             if not isinstance(node, list):
@@ -71,9 +82,7 @@ class DotDict(MutableMapping):
 
     def _set_reference(self, path, key):
         data = self._root
-        print(path)
         for item_no, item in enumerate(path):
-            print(item)
             if isinstance(item, int):
                 if not isinstance(data, list):
                     data = []
@@ -90,6 +99,10 @@ class DotDict(MutableMapping):
         data = self._root
         last = len(keys) - 1
         for pos, key in enumerate(keys):
+
+            if data is None:
+                return False
+
             if isinstance(key, int):
                 # Is int but dat is not list
                 if not isinstance(data, list):
@@ -104,7 +117,10 @@ class DotDict(MutableMapping):
 
             elif key not in data:
                 return False
-            data = data[key]
+            try:
+                data = data[key]
+            except TypeError:
+                return False
         return True
 
     def _reference(self, keys):
@@ -207,6 +223,7 @@ class DotDict(MutableMapping):
         return self._root
 
     def __setstate__(self, state):
+        self._override_data = False  # Use default value
         self._root = state
 
     def __iter__(self):
@@ -221,7 +238,7 @@ class DotDict(MutableMapping):
         else:
             return False
 
-    def __lshift__(self, list_of_kv) -> 'DotDict':
+    def __lshift__(self, list_of_kv: Union[List[Tuple[str, str]], Dict[str,str]]) -> 'DotDict':
         list_of_kv = list_of_kv.items() if isinstance(list_of_kv, dict) else list_of_kv
 
         for key, value in list_of_kv:
@@ -229,7 +246,7 @@ class DotDict(MutableMapping):
 
         return self
 
-    def __rshift__(self, list_of_kv: List[Tuple[str, str]]) -> 'DotDict':
+    def __rshift__(self, list_of_kv: Union[List[Tuple[str, str]], Dict[str,str]]) -> 'DotDict':
         list_of_kv = list_of_kv.items() if isinstance(list_of_kv, dict) else list_of_kv
 
         dot = DotDict()
@@ -239,6 +256,9 @@ class DotDict(MutableMapping):
             dot[key] = self[value]
         return dot
 
+    def __or__(self, property):
+        return self.get_or_none(property)
+
 def _data_convert(list_of_left_right):
     if isinstance(list_of_left_right, set):
         return [(item, item) for item in list_of_left_right]
