@@ -303,6 +303,34 @@ def test_invalid_paths_handling():
             _ = cd[path]
 
 
+def test_get_or_none():
+    d = {
+        "a": {"b": ["c", 0]},
+        "a3": {"b": ["c", 0, {"c"}, ("x", 10)]}
+    }
+
+    cd = DotDict(d)
+
+    assert (cd | 'a') == {"b": ["c", 0]}
+    assert (cd | "none") is None
+
+def test_reference():
+    d = {
+        "a": None,
+    }
+
+    cd = DotDict(d)
+    assert ('a' in cd) is True
+    assert ('a.b' in cd) is False
+
+
+def test_error_handling():
+    # Cannot assign string-key 'email' to non-dict: http://localhost/1
+    d = DotDict({'name': 'http://localhost/1'}, override_data=False)
+    d['name.my'] = {"a":"test"}
+    assert d['name.my'] == {"a":"test"}
+
 # Run all tests when executed via pytest
 if __name__ == "__main__":
     pytest.main()
+
